/*
 	This file is part of DataFox, a bandwidth usage monitor for 
 	BSNL DataOne broadband.
 	Author: Brian Fernandes - infernalproteus@gmail.com

    DataFox is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DataFox is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DataFox; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
var keyExp = new RegExp('.*key=([0-9]+).*', 'gi');
var locExp = new RegExp(".*\(\'(.*)\'\).*", 'gi');
var dataExp = new RegExp("<tr>([\\s\\S]+?)</tr>", "i");
var numExp = new RegExp(">[0-9]*\\.?[0-9]+", "gi");

DFox.prototype.bsnlLogin = function() {
	if (this.domain) {
		//make first contact		
		this.phase = this.FIRST_CONTACT;		
		talkToServer("http://" + this.domain, null, null, null, this);
	} else {
		this.phase = this.ZERO_CONTACT;		
		talkToServer("http://sancharnet.in/cgi-bin/redirect.pl", null, null, null, this, true);
	}
};


DFox.prototype.redirectIP = function(ip) {
	this.log("Redirected to " + ip);
	this.domain = ip;
//	this.logIn();
	this.phase = this.FIRST_CONTACT;
	this.responseHandler(null, null);
};


DFox.prototype.bsnlResponseHandler = function(data) {
	switch(this.phase) {		
		case this.FIRST_CONTACT:
			this.phase = this.LOGIN;
			var pData = "username=" + this.username + "&password=" + this.password;
			talkToServer("http://" + this.domain + "/secu/myportallogin.jsp", pData, null, null, this);
			break;
		case this.LOGIN:
			this.phase = this.SET_PARAMS;	
			//if the url contains loginerr...we have a problem
			if (data.indexOf("loginerr") != -1) {
			this.log("Site error");
				this.setState(this.ERROR);
				break;
			}
			keyExp.lastIndex = 0;
			var keyA = keyExp.exec(data);
			if (!keyA || keyA.length < 2) {
				this.log("Could not obtain key");
				this.setState(this.ERROR);
				break;
			}
			this.key = keyA[1];			
			talkToServer("http://" + this.domain + ":80/myportal/loginsetparameter.jsp?key="+this.key, null, null, null, this);
			break;
		case this.SET_PARAMS:
			this.phase = this.GET_SUPPLIES;			
			var gData = "key="+this.key+"&queryDate=" + this.dateString + "&username="+this.username+"&ispId=&idAndName=";
			var openURL = "http://" + this.domain + "/myportal/serviceResults.jsp?" + gData;
			this.eval(this.createEquateString("this.openURL", openURL));
			talkToServer(openURL, null, null, null, this);			
			break;				
		case this.GET_SUPPLIES:
			this.setState(this.LOGGEDIN);
			this.parseBSNLUsageData(data);
			break;	
	}
};

DFox.prototype.parseBSNLUsageData = function (aData) {
	this.log("Parsing usage data");
	//initialize all to N/A
    //dataFox.tSend = dataFox.tRecv = dataFox.tXfer = dataFox.tHours = dataFox.nXfer = "N/A";
    var upload, download, hours, xfer, nxfer;
    upload = download = hours = xfer = nxfer = "N/A";
    
    var temp = aData.split("Total Send Volume", 2);
    dataExp.lastIndex = 0;
    if (!temp || temp.length < 2) {
    	this.setState(this.ERROR, "Usage data N/A");
    	return;
    }
    
    var result = temp[1].match(dataExp);
    
    if (!result || result.length < 2) {
    	this.setState(this.ERROR, "Usage data N/A");
    	return;
    }
    
    numExp.lastIndex = 0;
    result = result[1].match(numExp);
    	
	//values always stored in KB	
	try {
		upload = parseFloat(result[0].substring(1)) * GiB_2_KiB; //tS    
	    download = parseFloat(result[1].substring(1)) * GiB_2_KiB; //tR
	    xfer = parseFloat(result[2].substring(1)) * GiB_2_KiB; //tX
	    hours = result[3].substring(1); //tH
	    nxfer = parseFloat(result[4].substring(1)) * GiB_2_KiB; //nX
	} catch (e) {
		this.log(e);
	}
	
	/*this.eval(this.createEquateString("this.tSend", upload, 
									  "this.tRecv", download, 
									  "this.tXfer", xfer, 
									  "this.nXfer", nxfer,
									  "this.tHours", hours,
									  "this.activeDateString", this.dateString));*/
									  
	var cProps = new Object();
	cProps["dtS"] = upload;
	cProps["dtR"] = download;
	cProps["dtX"] = xfer;
	cProps["dcX"] = cProps["dnX"] = nxfer;
	cProps["ntH"] = hours;
	
	this.eval(this.createEquateString("this.activeDateString", this.dateString, "this.cProps", cProps));
	this.refreshUI();
};