/*
 	This file is part of DataFox, a bandwidth usage monitor for 
 	BSNL DataOne broadband.
 	Author: Brian Fernandes - infernalproteus@gmail.com

    DataFox is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DataFox is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DataFox; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
//var gObserver = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
//var wm = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator);
var key;

//constants
var month = new Array("Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec");
var MAX_TT_LINES = 10;
var ROUND_TO = 1000;
var SEPARATOR = ",;,";
var GiB_2_KiB = 1024 * 1024;
var MIDNIGHT = -19800000; //1st Jan 1970, 0000 IST
var EIGHTAM = 9000000; //1st Jan 1970, 0800 IST

function DFox() {
	this.name = "chrome://datafox/";
	this.dfPrefs = new prefs();
	this.dfPrefs.initPrefs();
	this.dateString = getMonthString(new Date());
	this.pMgrI = Components.classes["@mozilla.org/passwordmanager;1"].getService(Components.interfaces.nsIPasswordManagerInternal);
	try {
		this.sandbox = Components.utils.Sandbox("http://www.thegoan.com/datafox");
	} catch (e) {
		this.log(e);
	}
	this.cProps = new Object();
	
	this.descCount = 0;
	this.descArray = new Array();
	for (var i = 0; i < MAX_TT_LINES; i++) this.descArray[i] = document.createElement("description");
	
	//phase constants
	this.ZERO_CONTACT = 0;
	this.FIRST_CONTACT = 1;
	this.LOGIN = 2;
	this.SET_PARAMS = 3;
	this.GET_SUPPLIES = 4;	
	this.GET_NEXT_PAGE = 5;
	
	//state constants
	this.ERROR = 0;
	this.INACTIVE = 1;
	this.LOGGINGIN = 2;
	this.LOGGEDIN = 3;
	
	//message constants
	this.SYNCH_REQ = 0;
	this.SYNCH = 1;
	this.SET_STATE = 2;
	this.EVAL = 3;
	this.REFRESH_UI = 4;
}

DFox.prototype.log = function(message) {
	if (!this.dfPrefs.getBoolPref(this.dfPrefs.DEBUG)) return;
	//if (!DEBUG) return;
	
	if (this._logService == null) {
	        this._logService = Components.classes['@mozilla.org/consoleservice;1'].getService();
	        this._logService.QueryInterface(Components.interfaces.nsIConsoleService);
	}
	
	this._logService.logStringMessage("datafox: " + message);
};


DFox.prototype.onLoad = function() {	
	this.log("DataFox Starting up");
	
	//widgets...
	this.ttMessage = document.getElementById("df-statusbar-tooltip-message");
	this.ttBold = document.getElementById("df-statusbar-tooltip-username");		
	this.statusBar = document.getElementById("df-statusbar");
	this.ttCustom = document.getElementById("df-statusbar-tooltip-custom");
	this.ttCustomLabel = document.getElementById("df-statusbar-tooltip-custom-label");
	this.dfLogout = document.getElementById("df-menuitem-logout");
	this.dfUpdate = document.getElementById("df-menuitem-update");
	this.dfBrowser = document.getElementById("df-menuitem-browser");
	this.dfBrowserAll = document.getElementById("df-menuitem-allmonths");
	this.dfMonth = document.getElementById("df-menuitem-month");
	this.dfContextMenu = document.getElementById("df-status-context");
	this.dfContextSeparator = document.getElementById("df-menuitem-separator");
	
	this.observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
	
	//are there any other observers around?
	var enumeration = this.observerService.enumerateObservers(this.name);
	var existing = enumeration && enumeration.hasMoreElements();
	var firstBrowser = existing ? enumeration.getNext() : null;
	
	//add observer
    this.observerService.addObserver(this, this.name, false);
		
	if (!existing) {
		//initialize to inactive state
		this.setState(this.INACTIVE);
		
		this.updateLoginDetails();
		
		if (this.dfPrefs.getBoolPref(this.dfPrefs.LOGIN_STARTUP)) {
			this.logIn();
		}
	} else {
		//ask for synch up from first observer
		var observer = firstBrowser.QueryInterface(Components.interfaces.nsIObserver);
		observer.observe(this, this.name, this.SYNCH_REQ + "");
	}
	

};

DFox.prototype.onUnload = function() {
	this.observerService.removeObserver(this, this.name);
};

DFox.prototype.update = function(prompt, aEvent) {
	 // on windows, right click calls this method
  	if (aEvent && (aEvent.button == 2)) return;
    
	if (this.insufficientLoginInfo()) {
		//insufficient data to log in 
		if (prompt) {
			window.openDialog('chrome://datafox/content/login.xul', '_blank', 'chrome,centerscreen,resizable=no,dependant=yes');
		}
		return;
	}	
	this.logIn();
};

DFox.prototype.insufficientLoginInfo = function() {
	return 	(!this.username ||
			(!this.password && this.isp != 1) || //password not required for MTNL Bombay
			(!this.domain && this.isp == 0 && this.isOverrideActive()));		
};

DFox.prototype.logout = function() {
	this.log("Logging out");
	this.setState(this.INACTIVE);		
};

DFox.prototype.isOverrideActive = function() {
	return this.dfPrefs.getBoolPref(this.dfPrefs.OVERRIDE_DOMAIN);
};

DFox.prototype.updateLoginDetails = function() {
	var username = this.dfPrefs.getCharPref(this.dfPrefs.ACTIVE_USER);
	if (username) {		
		this.password = this.getSavedPassword(username);		
	} else {
		this.password = null;
	}
	this.username = username;
	
	this.setISP(this.dfPrefs.getIntPref(this.dfPrefs.ACTIVE_ISP));
	this.log("ISP is " + this.isp);
	if (this.isp == 0 && this.isOverrideActive()) {
		this.log("IP override active");
		this.domain = this.dfPrefs.getCharPref(this.dfPrefs.ACTIVE_DOMAIN);
	}
};

DFox.prototype.getSavedPassword = function(username) {
	var unwanted = {value:""};
	var password = {value:""};
	try {
		this.pMgrI.findPasswordEntry(this.name, username, null, unwanted, unwanted, password);
	} catch(e) {
		//do nothing
	}
	if (password.value && password.value.length > 0) return password.value;
	return null;
};


DFox.prototype.logIn = function() {
	//not enough info to go on
	if (this.insufficientLoginInfo()) {
		this.log("Not enough info to log in");
		return;
	}
		
	this.log("Logging in with account " + this.username);
	
	//continue with the login process
	//change icon
	this.setState(this.LOGGINGIN, this.dateString);
	
	switch(this.isp) {
		case 0:
			this.bsnlLogin();
			break;
		case 1: 
			this.mtnlBOMLogin();
			break;
		case 2:		
			this.mtnlDELLogin();
			break;			
	}		
};

DFox.prototype.responseHandler = function(data, request) {	
	//cancelled request
	if (request && request.status == Components.results.NS_ERROR_ABORT) return;
	switch(this.isp) {
		case 0:
			this.bsnlResponseHandler(data);
			break;
		case 1: 
			this.mtnlBOMResponseHandler(data);
			break;
		case 2:
			this.mtnlDELResponseHandler(data);
			break;
	}	
};

DFox.prototype.setState = function(state, eMsg) {
	this.observerService.notifyObservers(this, 
										this.name, 
										this.SET_STATE + SEPARATOR + state + (eMsg ? (SEPARATOR + eMsg) : ""));
};

DFox.prototype.observe = function(subject, topic, data) {
	var parts = data.split(SEPARATOR);
	var msgType = parseInt(parts[0]);
	
	switch (msgType) {
		case this.SYNCH_REQ:
			this.log("SYNCH REQUEST RECEIVED");
			//collect all the data required
			var evString = this.createEquateString("this.username", this.username,
									"this.password", this.password,
									"this.domain", this.domain,
									"this.cProps", this.cProps,
								  	"this.activeDateString", this.activeDateString,
								  	"this.openURL", this.openURL
								  	);
			
			//add set isp, set state
			evString += "this.setISP(" + this.isp + ");";
			evString += "this.internalSetState(" + this.state + ");";
			evString += "this.updateStrings();";			
			this.observerService.notifyObservers(this, this.name, this.SYNCH + SEPARATOR + evString);									
//			subject.observe(this, this.name, this.SYNCH + SEPARATOR + evString);									
			break;
		case this.SYNCH:
			this.log(parts[1]);
			eval(parts[1]);
			break;
		case this.SET_STATE:
			this.internalSetState(parseInt(parts[1]), ((parts.length == 3) ? parts[2] : null));
			break;
		case this.EVAL:
			eval(parts[1]);
			break;
		case this.REFRESH_UI:
			this.updateStrings();
			break;
	}
};

DFox.prototype.internalSetState = function(state, eMsg) {
	this.state = state;
	switch(state) {
		case this.ERROR:
			if (!eMsg) eMsg = "Error logging in, please check your username, password & domain";
			this.updateUI("error",
							"DataFox: " + eMsg,
							"", 
							true,
							true,
							false,
							true);
			this.username = null;
			this.statusBar.label = "";
			break;
		case this.INACTIVE:
			this.updateUI("inactive",
							"Click to load",
							"",
							false,
							true,
							false,
							true);
			this.username = null;
			this.statusBar.label = "";
			if (httpChannel != null) {
				try {
			//		channel.cancel(Components.results.NS_ERROR_ABORT);
					httpChannel.cancel(Components.results.NS_ERROR_ABORT);
				} catch (e) {
				}				
			}
			break;
		case this.LOGGINGIN:
			this.updateUI("loading",
							"Loading " + eMsg,
							"",
							false,
							true,
							true,
							false);
			this.activeDateString = null;
			break;
		case this.LOGGEDIN:
			this.updateUI("active",
							"Logged in as",
							this.username,
							false,
							false,
							false,
							false);
			break;
	}
};

DFox.prototype.updateUI = function(status, message, bMessage, error, collapsed, uState, lState) {
	this.statusBar.setAttribute("df-status", status);
	this.ttMessage.value = message;
	this.ttBold.value = bMessage;
	this.ttMessage.setAttribute("error", error);
	this.ttCustom.collapsed = collapsed;
	this.dfUpdate.setAttribute("disabled", uState);
	this.dfLogout.setAttribute("disabled", lState);
	this.dfBrowser.setAttribute("disabled", uState);		
	this.dfMonth.setAttribute("disabled", uState);// || (this.isp == 1));
};

DFox.prototype.createCustomTooltip = function(parent, inString) {
	var parts = inString.split("\n");
	var i;
	for (i = 0; i < parts.length && i < MAX_TT_LINES; i++) {
		parent.appendChild(this.descArray[i]);
		this.descArray[i].value = parts[i];
	}
	
	while (i < this.descCount) {
		parent.removeChild(this.descArray[i++]);
	}
	
	this.descCount = parts.length;	
};

DFox.prototype.showTip = function(obj) {
	/*switch(this.phase) {
		case this.FIRST_CONTACT:
		case this.LOGIN:
		case this.SET_PARAMS:
			this.ttMessage.value = "Logging in...";
			break;
		case this.GET_SUPPLIES:
			this.ttMessage.value = "Logged in as ";
			this.ttBold.value = this.username;			
			break;
	}*/
};

DFox.prototype.setISP = function(value) {
	this.isp = value;
	//if MTNL Bombay or Delhi, remove Month menu and "All Months" selector
	if (this.isp >= 1) {
		if (this.dfMonth.parentNode) {
			this.dfContextMenu.removeChild(this.dfMonth);		
			this.dfBrowserAll.label = "Last Month";
			//this.dfBrowser.firstChild.removeChild(this.dfBrowserAll);
		}	
	} else {
		if (!this.dfMonth.parentNode) {
			this.dfContextMenu.insertBefore(this.dfMonth, this.dfContextSeparator);
			this.dfBrowserAll.label = "All Months";
			//this.dfBrowser.firstChild.appendChild(this.dfBrowserAll);
		}
	}
};

DFox.prototype.createMonthMenu = function(menupopup) {
	var date = new Date();
	var children = menupopup.childNodes;
   	for (var i = 0; i < children.length; i++) {
   		children[i].label = month[date.getMonth()] + " " + new String(date.getFullYear()).substring(2);
   		var dString = getMonthString(date);
		children[i].setAttribute("oncommand", "dataFox.selectMonth('" + dString + "')");
		children[i].setAttribute("checked", (dString == this.activeDateString)); 
   		date.setMonth(date.getMonth()-1);
   	}   	
};

function getBrowserForTab(aTab){
  var panel = document.getElementById(aTab.linkedPanel);
  return panel ? 
  			(panel.firstChild.nextSibling ?  panel.firstChild.nextSibling
  										  : panel.firstChild)
  			 : null;
}

DFox.prototype.openInBrowser = function(openRecords) {
	var url;
	var pd;
	if (this.isp == 0) {
		//bsnl
		url = openRecords ? ("http://" + this.domain + "/myportal/serviceRecords.jsp") 
						: this.openURL;
	} else if (this.isp == 1) {
		/*if (openRecords) {
			url = "http://register.mtnl.net.in/cgi-bin/display.cgi";
			pd = this.openURLPD2;
		} else {
			url = "http://register.mtnl.net.in/cgi-bin/sessdetr.cgi";
			pd = this.openURLPD;
		}
		this.log(url);
		this.log(pd);*/
		this.mtnlBOMOpenInBrowser(openRecords);
		return;
	} else if (this.isp == 2) {
		this.mtnlDELOpenInBrowser(openRecords);
		return;
	}
	var browser = getBrowserForTab(getBrowser().mCurrentTab);
	//is the current tab blank?
	this.log("Loading URL: " + url);
	if (!browser.sessionHistory || browser.sessionHistory.count < 2 &&
		(!browser.currentURI || browser.currentURI.spec == 'about:blank')) {		
		getBrowser().loadURI(url, null, null, convertToPostData(this.openURLPD));
	} else {		
		getBrowser().loadOneTab(url, null, null, convertToPostData(pd));
	}
};

DFox.prototype.selectMonth = function(dString) {
	this.dateString = dString;
	this.log("Selecting month: " + this.dateString);
	//this.statusBar.label = "Loading: " + dString;
	//this.setState(this.LOGGINGIN, this.dateString);
	this.update(true);
};

function getMonthString(date) {
	var mon = date.getMonth() + 1;
	return date.getFullYear() + "-" + (mon < 10 ? "0" : "") + mon;
}

function isBetween12_8(t1, t2) {
	if (t1.getDate() != t2.getDate()) return false;
	
	//set them to 1st Jan 1970, which is our reference date
	t1.setDate(1); 
	t1.setMonth(0);
	t1.setYear(1970);
	
	t2.setDate(1); 
	t2.setMonth(0);
	t2.setYear(1970);
	
	return (t1 >= MIDNIGHT && t2 <= EIGHTAM);
}


DFox.prototype.updateStrings = function() {
	//only if we're logged in 
	if (this.state != this.LOGGEDIN) return;
	
	//round the digits
	var vArray = this.valuesToString();
	
	//var sString = "{tX} / {nX}";
    var sString = dataFox.dfPrefs.getCharPref(dataFox.dfPrefs.STATUS_STRING);    
    this.statusBar.label = this.evalString(sString, vArray);
    
    //set the tooltip
	var tString = dataFox.dfPrefs.getCharPref(dataFox.dfPrefs.TOOLTIP_STRING);
	this.createCustomTooltip(this.ttCustom, this.evalString(tString, vArray));						
};

DFox.prototype.evalString = function(sString, vArray) {
	sString = this.formatString(sString, vArray);
    var out;
    try {
   		out = Components.utils.evalInSandbox(sString, this.sandbox);
   	} catch (e) {
   		this.log(e);
   	}
   	this.log(typeof(out));
   	if (typeof(out) != "string") {
   		out = sString;
   	}
   	return out;
}

DFox.prototype.formatString = function(sString, array) {
	for (prop in array) {
		sString = sString.replace(new RegExp("{" + prop.substring(1) + "}", "g"), array[prop]);		
	}	
	sString = sString.replace(/{YM}/g, this.dateString);
	return sString.replace(/{.{2}}/g, "N/A");	//replace any variables not found with "N/A"
};

DFox.prototype.createEquateString = function() {
	var evalString = "";
	for (var i = 0; i < arguments.length; i+=2) {
		var type = typeof(arguments[i+1]);
		if (type == "object") {
			//reset the object
			evalString += arguments[i] + "= new Object();";
			for (avar in arguments[i+1]) {
				evalString += arguments[i] + "['" + avar + "']=" + wrap(arguments[i+1][avar]) + ";";
			}			
		} else if (type != "undefined")  {
		 	evalString += arguments[i] + "=" + wrap(arguments[i+1]) + ";";
		}
	}
	return evalString;
};

function wrap(input) {
	return (typeof(input) == "string") ? ("'" + input + "'") : input;
}

DFox.prototype.eval = function(dataString) {
	this.observerService.notifyObservers(this, this.name, this.EVAL + SEPARATOR + dataString);
};

DFox.prototype.refreshUI = function() {
	this.observerService.notifyObservers(this, this.name, this.REFRESH_UI);	
};

DFox.prototype.valuesToString = function() {
	var mb = this.dfPrefs.getIntPref(this.dfPrefs.DATA_UNIT);
	var array = new Object();
	for (prop in this.cProps) {
		if (prop.length == 3 && prop[0] == "d") {
			//we've got to round and convert this value, it's a data value
			array[prop] = RAC(this.cProps[prop], mb);
		} else {
			array[prop] = this.cProps[prop];
		}
	}
	return array;
};

function RAC(value, mb) {
	value /= 1024; 			//convert to MiB
	if (!mb) value /= 1024; //convert to GiB
	return Math.round(ROUND_TO * value) / ROUND_TO;	
}

var dataFox = new DFox();

function onLoad() {
	dataFox.onLoad();
}

function onUnload() {
	dataFox.onUnload();
}

window.addEventListener("load", onLoad, false);
window.addEventListener("unload", onUnload, false);
