/*
 	This file is part of DataFox, a bandwidth usage monitor for 
 	BSNL DataOne broadband.
 	Author: Brian Fernandes - infernalproteus@gmail.com
 	This file contains adapted versions of functions from the Gmail Notifer 
 	extension by Doron Rosenberg. 

    DataFox is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DataFox is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DataFox; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
var nsIIOService = Components.interfaces.nsIIOService;
var kIOSERVICE_CONTRACTID = "@mozilla.org/network/io-service;1";

var httpChannel;
var channel;

function talkToServer(aURL, aPostData, aCookieData, aReferrer, dfInstance, redirect, timeout) {
  dataFox.log(aURL);
  // the IO service
  var ioService = Components.classes[kIOSERVICE_CONTRACTID].getService(nsIIOService);
  // create an nsIURI
  var uri = ioService.newURI(aURL, null, null);
  //nsIInputStream
  var uploadStream = Components.classes["@mozilla.org/io/string-input-stream;1"]
    .createInstance(Components.interfaces.nsIStringInputStream);
  if (aPostData)
    uploadStream.setData(aPostData, aPostData.length);

  channel = ioService.newChannelFromURI(uri);
  // get a httpchannel and make it a post
  httpChannel = channel.QueryInterface(Components.interfaces.nsIHttpChannel);
  // set a referrer
  if (aReferrer) {
    var referrerUri = ioService.newURI(aReferrer, null, null);
    httpChannel.referrer = referrerUri;
  }
  if (aPostData) {
    var uploadChannel = channel.QueryInterface(Components.interfaces.nsIUploadChannel);
    uploadChannel.setUploadStream(uploadStream, "application/x-www-form-urlencoded", -1);

    // order important - setUploadStream resets to get/put
    httpChannel.requestMethod = "POST";
  }
 if (aCookieData){
    httpChannel.setRequestHeader("Cookie", aCookieData, false);
    /*for (var run = 0; run < aCookieData.length; run++){
      httpChannel.setRequestHeader("Cookie", aCookieData[run], true);
    }*/
  }
  
  if (dfInstance) {
	  var observer;
	  if (!redirect) {
	  	observer = new httpObserver(dfInstance);
	  	/*if (timeout) {
	  		setTimeout("httpChannel.cancel(Components.results.NS_ERROR_ABORT);", timeout);
	  	}*/
	  } else {
	  	observer = new redirectObserver(dfInstance);
		channel.notificationCallbacks = observer;		  
	  }
	  return channel.asyncOpen(observer, null);
  } else {
  	  return channel.open();
  }	  
}


function getData(stream) {
	var scriptableInputStream = 
        Components.classes["@mozilla.org/scriptableinputstream;1"]
          .createInstance(Components.interfaces.nsIScriptableInputStream);
    scriptableInputStream.init(stream);
    var data = "";
    while (stream.available()) {
    	data += scriptableInputStream.read(stream.available());
    }
//    return scriptableInputStream.read(stream.available());	
	return data;
}

function convertToPostData(dataString) {
	// POST method requests must wrap the encoded text in a MIME
	// stream
	if (!dataString) return null;
	
	const Cc = Components.classes;
	const Ci = Components.interfaces;
	var stringStream = Cc["@mozilla.org/io/string-input-stream;1"].
	                   createInstance(Ci.nsIStringInputStream);
	stringStream.setData(dataString, dataString.length);
	
	var postData = Cc["@mozilla.org/network/mime-input-stream;1"].
	               createInstance(Ci.nsIMIMEInputStream);
	postData.addHeader("Content-Type", "application/x-www-form-urlencoded");
	postData.addContentLength = true;
	postData.setData(stringStream);
	
	return postData;
}


var redirectObserver = function(dfInstance) {
	return ({
		newIP : "",
		onStartRequest : function (aRequest, aContext) {
	    },
	    onDataAvailable : function (aRequest, aContext, aStream, aSourceOffset, aLength){
	    },
	    onStopRequest : function (aRequest, aContext, aStatus) {
	    	if (this.newIP) {
	    		dfInstance.redirectIP(this.newIP);			
	    	} else {
	    		//probably no internet connection or redirect failure
	    		dfInstance.log("Could not obtain server IP"); 
	    		dfInstance.setState(dfInstance.ERROR, "Error obtaining server IP. Check internet connection/use IP override");
	    	}
	    },    
		onChannelRedirect: function (aOldChannel, aNewChannel, aFlags) {
			this.newIP = aNewChannel.URI.host;
		},	  
		onRedirect : function (aOldChannel, aNewChannel) {
		},
		onProgress : function (aRequest, aContext, aProgress, aProgressMax) { 
		},
	  	onStatus : function (aRequest, aContext, aStatus, aStatusArg) { 
	  	},
		getInterface: function(aIID) {
		   return this;
	  	}
  	});
}


var httpObserver = function(dfInstance, timer) {
  return ({
    data : "",	
	
    onStartRequest : function (aRequest, aContext) {
		this.data = "";
    },

    onDataAvailable : function (aRequest, aContext, aStream, aSourceOffset, aLength){
      var scriptableInputStream = 
        Components.classes["@mozilla.org/scriptableinputstream;1"]
          .createInstance(Components.interfaces.nsIScriptableInputStream);
      scriptableInputStream.init(aStream);

      this.data += scriptableInputStream.read(aLength);
    },

    onStopRequest : function (aRequest, aContext, aStatus) {
		dfInstance.responseHandler(this.data, aRequest);
    },     
    
    cancel : function() {
    	alert('hero');
    	dfInstance.log("cancelled");
    },

    QueryInterface : function(aIID) {
      if (aIID.equals(Components.interfaces.nsIStreamListener) ||
          aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
          aIID.equals(Components.interfaces.nsIAlertListener));
        return this;
      throw Components.results.NS_NOINTERFACE;
    }
  }
  );
}

function getBrowserForTab(aTab){
  var panel = document.getElementById(aTab.linkedPanel);
  return panel ? 
  			(panel.firstChild.nextSibling ?  panel.firstChild.nextSibling
  										  : panel.firstChild)
  			 : null;
}

function openInBrowser(url, postData) {
	var currentTab = getBrowser().mCurrentTab;
	var browser = getBrowserForTab(currentTab);
	//is the current tab blank?
	var reusable = (!browser.sessionHistory || browser.sessionHistory.count < 2 &&
		(!browser.currentURI || browser.currentURI.spec == 'about:blank') &&
		!currentTab.hasAttribute("busy"));
	if (reusable && !postData) {
		getBrowser().loadURI(url);
	} else {
		var tab = getBrowser().loadOneTab(url, null, null, convertToPostData(postData));
		if (reusable && tab)  getBrowser().removeAllTabsBut(tab);
	}
}
