/*
 	This file is part of DataFox, a bandwidth usage monitor for 
 	BSNL DataOne broadband.
 	Author: Brian Fernandes - infernalproteus@gmail.com

    DataFox is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DataFox is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DataFox; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
var pMgr = Components.classes["@mozilla.org/passwordmanager;1"].getService(Components.interfaces.nsIPasswordManager);

var dataFox = window.opener.dataFox;
var dfPrefs = dataFox.dfPrefs;
var usernameW;
var passwordW;
var savepwW;
var domainLW;
var domainW;
var overrideW;
var ispW;
var rowsW;
var overrideRowW;
var domainRowW;
var passwordRowW;
var storePasswordRowW;
var titleW;

var host = "chrome://datafox/";


function onLoad() {	
	usernameW = document.getElementById("username");
	passwordW = document.getElementById("password");
	savepwW = document.getElementById("store-password");
	overrideW = document.getElementById("override-domain");
	domainLW = document.getElementById("domain-label");
	domainW = document.getElementById("domain");
	rowsW = document.getElementById("layout-rows");
	passwordRowW = document.getElementById("password-row");
	storePasswordRowW = document.getElementById("store-password-row");
	overrideRowW = document.getElementById("override-row");
	domainRowW = document.getElementById("domain-row");
	titleW = document.getElementById("login-title");
	ispW = document.getElementById("isp");
	
	//populate the user list
	populateUserList();
	
	//load username and password	
	var lastUser = dfPrefs.getCharPref(dfPrefs.ACTIVE_USER);
	if (lastUser) {	
		usernameW.value = lastUser;		
		var password = dataFox.getSavedPassword(lastUser);
		if (password && password.length > 0) {
			passwordW.value = password;
		}
	}
	
	//load "save password"
	savepwW.checked = dfPrefs.getBoolPref(dfPrefs.SAVE_PASSWORD);
	
	//load domain details
	overrideW.checked = dfPrefs.getBoolPref(dfPrefs.OVERRIDE_DOMAIN);
	
	var domain = dfPrefs.getCharPref(dfPrefs.ACTIVE_DOMAIN);
	if (domain) {
		domainW.value = domain;
	}
	
	//load active isp
	var isp = dfPrefs.getIntPref(dfPrefs.ACTIVE_ISP);
	ispW.selectedIndex = isp;
	ispChanged(isp);
	
	overrideChanged();
}

function populateUserList() {
	var enumerator = pMgr.enumerator;
	while(enumerator.hasMoreElements()) {
		var np;
		try {
			np = enumerator.getNext();
		} catch(e) {
			break;
		}
		
		np = np.QueryInterface(Components.interfaces.nsIPassword);
		if (np.host == host) {
			try {
				var user = np.user;
				usernameW.appendItem(user, user);
			} catch (e) {				
			}
		}
	}
}

function ispChanged(selectedIndex) {
	var changed = false;
	switch (selectedIndex) {
		case 0:
			//BSNL
			if (!passwordRowW.parentNode) {
				rowsW.appendChild(passwordRowW);
				rowsW.appendChild(storePasswordRowW);				
				changed = true;
			}
			if (!overrideRowW.parentNode) {
				rowsW.appendChild(overrideRowW);
				rowsW.appendChild(domainRowW);
				changed = true;
			}
			titleW.setAttribute("bsnl", true);			
			break;
		case 1:
			//MTNL Bombay
			if (passwordRowW.parentNode) {
				rowsW.removeChild(passwordRowW);
				rowsW.removeChild(storePasswordRowW);
				changed = true;
			}
			//falls through
		case 2:
			titleW.setAttribute("bsnl", false);
			//MTNL Delhi
			if (selectedIndex == 2 && !passwordRowW.parentNode) {
				rowsW.appendChild(passwordRowW);
				rowsW.appendChild(storePasswordRowW);				
				changed = true;
			}
			if (overrideRowW.parentNode) {
				rowsW.removeChild(overrideRowW);
				rowsW.removeChild(domainRowW);
				changed = true;
			}
			break;
	}
	if (changed) window.sizeToContent();
}

function overrideChanged() {
	domainW.disabled = domainLW.disabled = !overrideW.checked;
}

function loadPassword() {
	var password = dataFox.getSavedPassword(usernameW.value);
	if (password && password.length > 0) {
		passwordW.value = password;
	}	
}

function onCancel() {
	dataFox.log("Login cancelled");
	dataFox.setState(dataFox.INACTIVE);
}

function onAccept() {	
	var isp = ispW.selectedIndex;
	var evalString = "this.setISP(" + isp + ");";
	dfPrefs.setIntPref(dfPrefs.ACTIVE_ISP, isp);
	
//	dataFox.username = usernameW.value;	
	evalString += "this.username='" + usernameW.value + "';";
	dfPrefs.setCharPref(dfPrefs.ACTIVE_USER, usernameW.value);
	
	//manage the password store	
	if (isp != 1) {
		try {
			pMgr.removeUser(host, usernameW.value);
		} catch (e) {
			//do nothing
		}
			
		pMgr.addUser(host, usernameW.value, savepwW.checked ? passwordW.value : "");	
		
		dfPrefs.setBoolPref(dfPrefs.SAVE_PASSWORD, savepwW.checked);
		evalString += "this.password='" + passwordW.value + "';";
		if (isp == 0) {
			dfPrefs.setCharPref(dfPrefs.ACTIVE_DOMAIN, domainW.value);
			dfPrefs.setBoolPref(dfPrefs.OVERRIDE_DOMAIN, overrideW.checked);
			if (overrideW.checked) {
//				dataFox.domain = domainW.value;
				evalString += "this.domain='" + domainW.value + "';";
			} else {
				evalString += "this.domain=null;";
			}
		}
	} else {
		//just store the username
		pMgr.addUser(host, usernameW.value, "");
	}
	
	dataFox.log(evalString);
	dataFox.eval(evalString);
	dataFox.update(false);	
	return true;
}
