/*
 	This file is part of DataFox, a bandwidth usage monitor for 
 	BSNL DataOne broadband.
 	Author: Brian Fernandes - infernalproteus@gmail.com
 	This file contains adapted versions of functions from the Gmail Notifer 
 	extension by Doron Rosenberg. 

    DataFox is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DataFox is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DataFox; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
function prefs() {

	this.prefBranch = null;
	this.DEBUG = "datafox.debug.active";
	this.ACTIVE_USER = "datafox.active.user";
	this.ACTIVE_DOMAIN = "datafox.active.domain";
	this.ACTIVE_ISP = "datafox.active.isp";
	this.OVERRIDE_DOMAIN = "datafox.override.domain";
	this.SAVE_PASSWORD = "datafox.save.password";
	this.TOOLTIP_STRING = "datafox.tooltip.string";
	this.STATUS_STRING = "datafox.status.string";
	this.DATA_UNIT = "datafox.data.unit";
	this.LOGIN_STARTUP = "datafox.login.startup";
	
}

prefs.prototype.getPrefBranch = function() {
	if (!this.prefBranch) {
		this.prefBranch = Components.classes['@mozilla.org/preferences-service;1'].getService();
		this.prefBranch = this.prefBranch.QueryInterface(Components.interfaces.nsIPrefBranch);
  	}
  	return this.prefBranch;
};

prefs.prototype.setCharPref = function(name, value) {
	this.getPrefBranch().setCharPref(name, value);
};

prefs.prototype.getCharPref = function(name) {
	var value = null;
	try {
		value = this.getPrefBranch().getCharPref(name);
		if (value && value.length == 0) value = null;		
	} catch (e) {
	}
	return value;	
};

prefs.prototype.setBoolPref = function(name, value) {
	this.getPrefBranch().setBoolPref(name, value);
};

prefs.prototype.getBoolPref = function(name) {
	var value = false;
	try {
		value = this.getPrefBranch().getBoolPref(name);
	} catch (e) {
	}
	return value;	
};

prefs.prototype.setIntPref = function(name, value) {
	this.getPrefBranch().setIntPref(name, value);
};

prefs.prototype.getIntPref = function(name) {
	var value = 0;
	try {
		value = this.getPrefBranch().getIntPref(name);
	} catch (e) {
	}
	return value;	
};


prefs.prototype.initPrefs = function() {
	this.initPref(this.ACTIVE_DOMAIN, "char", "10.240.144.195");	
	this.initPref(this.TOOLTIP_STRING, "char", "Up: {tS}GB Down: {tR}GB\nChargeable transfer: {cX}GB\nHours: {tH} Period: {YM}");
	this.initPref(this.STATUS_STRING, "char", "{cX}GB");
	this.initPref(this.DEBUG, "bool", false);
	this.initPref(this.ACTIVE_ISP, "int", 0);
	this.initPref(this.DATA_UNIT, "int", 0);
	this.initPref(this.LOGIN_STARTUP, "bool", true);
};

prefs.prototype.initPref = function(name, type, dValue) {
	switch(type) {
		case "char":
			var ePref = this.getCharPref(name);
			if (!ePref || ePref.length == 0) {
				this.setCharPref(name, dValue);
			}
			break;
		case "bool":
      		var ePref = this.getBoolPref(name);
      		if (ePref == null) {
        		this.setBoolPref(name, dValue);
        	}
      		break;
      	case "int":
      		var ePref = this.getIntPref(name);
      		if (!ePref) {
        		this.setIntPref(name, dValue);
        	}
      		break;      	
	}
};